# -*- coding: utf-8 -*-
from __future__ import unicode_literals
try:
    import re2 as re
except ImportError:
    import re

flags = re.S | re.I
PLACEHOLDER = '<@!hmin_placeholder_%s_!@>'
RE_REPLACED_TAG = re.compile(
    r'(<(script|textarea|style|pre).*?>.*?</(script|textarea|style|pre)>)',
    flags
)
RE_COMMENTS = re.compile(r'<!--(?!\[if.*?\]).*?-->', flags)
RE_PLACEHOLDER = re.compile(
    r'%s' % PLACEHOLDER.replace('%s', '([0-9]+)')
)


def minify(content, remove_comments=True):
    # helpers
    def tag_replace(m):
        safe_storage[m.start()] = m.group(1)
        return PLACEHOLDER %  m.start()

    def tag_return(m):
        return safe_storage[int(m.group(1))]

    # decode unicode
    try:
        content = content.decode('utf8')
    except (UnicodeDecodeError, UnicodeEncodeError):
        pass

    # replace dangerous tags with placeholders
    safe_storage = dict()
    content = RE_REPLACED_TAG.sub(tag_replace, content)

    # fast version of "\s{2,} to \s"
    content = ' '.join(content.split())
    # "strip_space_between_tags"
    content = content.replace('> <', '><')
    if remove_comments:
        content = RE_COMMENTS.sub('', content)

    # and return dangerous tags back
    return RE_PLACEHOLDER.sub(tag_return, content).strip()



# -*- coding: utf-8 -*-
from django.conf import settings
try:
    from django.core.cache import caches, InvalidCacheBackendError
except ImportError:
    from django.core.cache import get_cache, InvalidCacheBackendError
try:
    import xxhash
    hash_func = xxhash.xxh64
except ImportError:
    import hashlib
    hash_func = hashlib.md5
try:
    import re2 as re
except ImportError:
    import re
try:
    from django.utils.deprecation import MiddlewareMixin
except ImportError:
    # Not required for Django <= 1.9, see:
    # https://docs.djangoproject.com/en/1.10/topics/http/middleware/#upgrading-pre-django-1-10-style-middleware
    MiddlewareMixin = object
from .base import minify


""" Minification html middleware
"""
ENABLED = getattr(settings, 'HTML_MINIFY', not settings.DEBUG)
REMOVE_COMMENTS = getattr(settings, 'HMIN_REMOVE_COMMENTS', True)
USE_CACHE = getattr(settings, 'HMIN_USE_CACHE', True)
TIMEOUT = getattr(settings, 'HMIN_CACHE_TIMEOUT', 3600)
EXCLUDE = []

# get cache provider, or disable caching
cache_back = getattr(settings, 'HMIN_CACHE_BACKEND', 'default')
try:
    try:
        cache = caches[cache_back]
    except NameError:
        cache = get_cache(cache_back)
except InvalidCacheBackendError:
    USE_CACHE = False


# process exclude pages
if hasattr(settings, 'HMIN_EXCLUDE'):
    for url_pattern in settings.HMIN_EXCLUDE:
        regex = re.compile(url_pattern)
        EXCLUDE.append(regex)


class MarkMiddleware(MiddlewareMixin):
    def process_request(self, request):
        request.need_to_minify = True


class MinMiddleware(MiddlewareMixin):
    def process_response(self, request, response):
        # prevent from minifying cached pages
        if not hasattr(request, 'need_to_minify') or\
           not request.need_to_minify:
            return response

        # prevent from minifying excluded pages
        if EXCLUDE:
            path = request.path.lstrip('/')
            for regex in EXCLUDE:
                if regex.match(path):
                    return response

        if 'Content-Type' in response and\
                'text/html' in response['Content-Type'] and ENABLED:
            if USE_CACHE:
                key = 'hmin_%s' % hash_func(response.content).hexdigest()
                data = cache.get(key)
                if data:
                    response.content = data
                else:
                    response.content = minify(response.content, REMOVE_COMMENTS)
                    cache.set(key, response.content, TIMEOUT)
            else:
                response.content = minify(response.content, REMOVE_COMMENTS)
        return response



# -*- coding: utf-8 -*-
import codecs
from os.path import abspath, dirname, join
from django.test import SimpleTestCase
from django.test.utils import override_settings

from hmin import minify


data_path = abspath(join(dirname(__file__), 'data'))
load_file = lambda name: codecs.open(
    '%s.html' % join(data_path, name), encoding='utf-8'
).read()


class MinifyTestCase(SimpleTestCase):
    def test_data_htmls(self):
        examples = [
            'habrahabr',
            'lenta',
            'gazeta',
            'youtube',
        ]
        for example in examples:
            print 'Test file %s' % example
            self.assertEqual(
                minify(load_file(example)), load_file(example + '_min')
            )



# -*- coding: utf-8 -*-
from functools import wraps

from .base import minify


def minify_plain(remove_comments=True):
    """ Minifies any function output

    Usage:
        @minify_plain()
        def my_cool_func():
            <...>
            return some_plain_html
    """
    def compress(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            return  minify(func(*args, **kwargs), remove_comments)
        return wrapper
    return compress


def minify_disable(func):
    """ Disable page minification

    Usage:
        @minify_disable
        def my_cool_func():
            <...>
            return some_plain_html
    """
    @wraps(func)
    def wrapper(*args, **kwargs):
        response = func(*args, **kwargs)
        response.need_to_minify = False
        return response
    return wrapper



# -*- coding: utf-8 -*-
import os
from setuptools import setup, find_packages
from hmin import __version__


# readme
descr = 'Fast HTML minification function, django middleware, decorator'

try:
    readme = open(
        os.path.join(
            os.path.dirname(os.path.abspath(__file__)), 'README.md'
        )
    ).read()
except IOError:
    readme = descr

try:
    import pypandoc
    readme = pypandoc.convert(readme, 'rst', format='md')
except:
    pass


setup(
    name='django-hmin',
    url='https://github.com/xfenix/django-hmin',
    version=__version__,
    description=descr,
    long_description=readme,
    author='Xfenix',
    author_email='ad@xfenix.ru',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'xxhash',
    ]
)
